/****************************************************************************
 *                                                                          
 * Title: Simple Scan Measurement Example
 *
 * Note: This example is designed for use with LabVINDOWS/CVI 7.0
 *
 * Purpose:
 *
 *  --- Scan with Transfer of Block data (suitable for ESPI and ESCI) ---
 *
 *  1. Programming of Scan parameters
 *        2 subscans
 *        use parameters of default setting, but program them to the unit
 *        scan start, scan stop, step mode
 *        subscan: start, stop, rbw, meas time, auto ranging, rf attn, (no preamp - is optional for ESPI)
 *        trace 1 clr write, detector peak; trace 2 clr write detector average single scan
 *
 *  2. Prepare scan
 *        enable srq with scan data available (STATus:OPERation) (workarounded)
 *        enable block transfer (TRACe:FEED:CONTrol ALWays)
 *        start scan
 *
 *  3. Data block transfer
 *        wait for SRQ (workarounded)
 *        read scan block
 *        evaluate level values from scan block
 *        display level values - needs to be discussed what is possible
 *
 *  --- Time Domain Scan with Transfer of Block data (suitable for ESCI) ---
 *
 *  1. Programming of Scan parameters
 *        select Time Domain
 *        measurement time 5 ms, total scan time 100 sec
 *        trace 1 clr write, detector peak; trace 2 clr write detector quasi peak
 *        single scan
 *
 *  2. Prepare scan
 *        enable srq with scan data available (STATus:OPERation) (workarounded)
 *        enable block transfer (TRACe:FEED:CONTrol ALWays)
 *        start scan
 *
 *  3. Data block transfer
 *        wait for SRQ (workarounded)
 *        read scan block
 *        evaluate level values from scan block
 *        display level values
 *
 * Note:
 *
 *  The input parameters are hardcoded in the source code.
 *
 ****************************************************************************/

/* --- LabVINDOWS/CVI specific libraries --- */
#include <userint.h>
#include <utility.h>
/* --- Common C libraries, Instrument driver library, VISA library (included in instrument driver) --- */
#include <ansi_c.h>
#include <visa.h>
#include "rsesci.h"
#include "rsesci_scan_measurement.h"

/*= Global Variable Definitions =============================================*/

#define ARRAY_SIZE  100000 // Maximal number of measured frequencies is limited to 100000 (per detector)

/* Error check macro for driver functions */
#define CHECKERR( fCal ) \
    { \
        char    BuFfEr[512];\
        error = (fCal);\
        rsesci_error_message (io, error, error_message);\
        if (error < VI_SUCCESS) {\
            sprintf (BuFfEr, "Primary Error: 0x%08X, %s\n", error, error_message);\
            MessagePopup ("Error", BuFfEr); \
            rsesci_error_query (io, &error, error_message);\
            sprintf (BuFfEr, "Secondary Error: 0x%08X, %s\n", error, error_message);\
            SetWaitCursor (0); \
            MessagePopup ("Error", BuFfEr); \
            runState = 0; \
            HidePanel (panelHandle[MEAS]); \
            DisplayPanel (panelHandle[CONFIG]); \
            break; \
        } \
    }

/* Globals */

ViInt32     runState            = 0,
            oldTraceMode        = 0,
            uirCode             = 0,
            panelHandle[10];

ViInt32     traceStatus         = 0,
            noofPoints          = 0,
            points              = 0,
            firstSubPoints      = 0,
            secondSubPoints     = 0,
            idx                 = 0;

ViChar      *statusData, *tempStatusData;
ViReal64    *T1, *tempT1;
ViReal64    *T2, *tempT2;
ViReal64    *T3, *tempT3;

ViReal64    *xArray,
            xIncrement          = 0.0;

ViReal64    xStart              = 0.0,
            xStop               = 0.0,
            xSubStart           = 0.0;

ViChar      buffer[256];
ViBoolean   domain              = VI_TRUE;

ViInt32     measurement         = VI_FALSE,
            traceDetector_1     = 0,
            traceDetector_2     = 0,
            traceMode_1         = 0,
            traceMode_2         = 0;

/* Define remote connection parameters (as default values) */

ViStatus    error               = VI_SUCCESS;
ViChar      error_message[512];
ViSession   io;

/* Misc */

ViInt32     stepMode        = 0;
ViBoolean   autoRanging     = VI_FALSE;

/*=================== Utility Function Declarations =========================*/
long abortApplication (void);
void memoryCleanup(void);

/*= Main Function ===========================================================*/

void main ()
{
    /* Load panels.*/
    panelHandle[BCKGRND] = LoadPanel (panelHandle[BCKGRND], "rsesci_scan_measurement.uir", BCKGRND);
    panelHandle[MEAS] = LoadPanel (panelHandle[BCKGRND], "rsesci_scan_measurement.uir", MEAS);
    panelHandle[CONFIG] = LoadPanel (panelHandle[BCKGRND], "rsesci_scan_measurement.uir", CONFIG); 
    panelHandle[INIT] = LoadPanel (panelHandle[BCKGRND], "rsesci_scan_measurement.uir", INIT);
    /* Display panels.*/
    DisplayPanel (panelHandle[BCKGRND]);
    DisplayPanel (panelHandle[INIT]);
    SetSleepPolicy (VAL_SLEEP_MORE);
    SetBreakOnLibraryErrors (0);

    uirCode = RunUserInterface ();
}   

/*===========================================================================*/
/* Function: Initialize Instrument                                           */
/* Purpose:  This is a callback function of the Continue button on the       */
/*           Initialize panel. It initializes the instrument and switches to */
/*           the panel Configure.                                            */
/*===========================================================================*/
int CVICALLBACK initInstrument (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViChar instrDescr[256], errorMessage[256], errBuffer[256];
    ViInt16 rst, id;

    switch (event) {
        case EVENT_COMMIT:
            SetWaitCursor(0);
            GetCtrlVal (panelHandle[INIT], INIT_ADDRESS, instrDescr);
            GetCtrlVal (panelHandle[INIT], INIT_ID, &id);
            GetCtrlVal (panelHandle[INIT], INIT_RST, &rst);
            SetWaitCursor(1);
            if ((error = rsesci_init (instrDescr, id, rst, &io)) < 0 )
                {
                rsesci_error_message(VI_NULL, error, errorMessage);
                sprintf (errBuffer, "Initialization Error:\n\n%s\n\nCheck your "
                        "connections and make sure you have the right instrument descriptor.",
                         errorMessage);
                MessagePopup ("ERROR!", errBuffer);
                SetWaitCursor(0);
                return(0);   
                }

            /* --- Application use preconditions --- */

            CHECKERR (rsesci_SetDisplayUpdate (io, VI_TRUE));
            CHECKERR (viSetAttribute (io, VI_ATTR_TMO_VALUE, 1000));
            CHECKERR (rsesci_SetLevelAxisUnit (io, RSESCI_SCREEN_A, RSESCI_UNIT_DBM));

            /* --- Memory alloc --- */

            tempStatusData = malloc (ARRAY_SIZE * sizeof (ViChar));
            statusData = malloc (ARRAY_SIZE * sizeof (ViChar));

            T1 = malloc (ARRAY_SIZE * sizeof (ViReal64));
            tempT1 = malloc (ARRAY_SIZE * sizeof (ViReal64));

            T2 = malloc (ARRAY_SIZE * sizeof (ViReal64));
            tempT2 = malloc (ARRAY_SIZE * sizeof (ViReal64));

            T3 = malloc (ARRAY_SIZE * sizeof (ViReal64));
            tempT3 = malloc (ARRAY_SIZE * sizeof (ViReal64));

            xArray = malloc (ARRAY_SIZE * sizeof (ViReal64));

            /* --- */

            SetWaitCursor(0);
            HidePanel (panelHandle[INIT]);
            DisplayPanel (panelHandle[CONFIG]);
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","This button causes the instrument to be initialized.");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Configure Measurement                                           */
/* Purpose:  This is a callback function of the Continue button on the       */
/*           Configure panel (configure & measure).                          */
/*===========================================================================*/
int CVICALLBACK configMeasurement (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            runState = 0;
            SetWaitCursor(0);
            HidePanel (panelHandle[CONFIG]);
            DisplayPanel (panelHandle[MEAS]);
            DeleteGraphPlot (panelHandle[MEAS], MEAS_GRAPH, -1, VAL_IMMEDIATE_DRAW);

            SetWaitCursor(1);

            // ------ Programming of Scan parameters ------

            CHECKERR (rsesci_SetInstrumentMode (io, RSESCI_INSTRUMENT_MODE_RECEIVER));

            GetCtrlVal (panelHandle[CONFIG], CONFIG_DOMAIN, &domain);
            
            if (domain == VI_TRUE)
                {
                // --- Scan in the Frequency Domain

                SetCtrlAttribute (panelHandle[MEAS], MEAS_GRAPH, ATTR_XMAP_MODE, VAL_LOG);
    
                // Set sweep frequency mode
                CHECKERR (rsesci_SetFrequencyMode (io, RSESCI_SCREEN_A, RSESCI_FREQ_SWEEP));
                // Scan Start
                CHECKERR (rsesci_SetStartFrequency (io, RSESCI_SCREEN_A, 150.0e3));
                // Scan Stop
                CHECKERR (rsesci_SetStopFrequency (io, RSESCI_SCREEN_A, 1.0e9));
                // Step Mode
                CHECKERR (rsesci_SetSweepSpacing (io, RSESCI_SCREEN_A, RSESCI_AUTO));
                }
            else
                {
                // --- Scan in the Time Domain

                SetCtrlAttribute (panelHandle[MEAS], MEAS_GRAPH, ATTR_XMAP_MODE, VAL_LINEAR);

                // Set fixed frequency mode
                CHECKERR (rsesci_SetFrequencyMode (io, RSESCI_SCREEN_A, RSESCI_FREQ_FIXED));
                // Set Total Scan Time
                CHECKERR (rsesci_SetScanTimeDomain (io, 100.0));
                // Set Receiver Frequency
                CHECKERR (rsesci_SetCenterFrequency (io, RSESCI_SCREEN_A, 1.0e9));
                // Set Measurement Time
                CHECKERR (rsesci_SetSweepTime (io, RSESCI_SCREEN_A, 5.0e-3));
                }

            // Trace Parameters

            GetCtrlVal (panelHandle[CONFIG], CONFIG_TRACE_DETECTOR_1, &traceDetector_1);
            GetCtrlVal (panelHandle[CONFIG], CONFIG_TRACE_DETECTOR_2, &traceDetector_2);
            GetCtrlVal (panelHandle[CONFIG], CONFIG_TRACE_MODE_1, &traceMode_1);
            GetCtrlVal (panelHandle[CONFIG], CONFIG_TRACE_MODE_2, &traceMode_2);
            //
            CHECKERR (rsesci_SetTraceMode (io, RSESCI_SCREEN_A, 1, traceMode_1));
            CHECKERR (rsesci_SetTraceDetector (io, RSESCI_SCREEN_A, 1, traceDetector_1));
            //
            CHECKERR (rsesci_SetTraceMode (io, RSESCI_SCREEN_A, 2, traceMode_2));
            CHECKERR (rsesci_SetTraceDetector (io, RSESCI_SCREEN_A, 2, traceDetector_2));

            // --- Scan Table ---

            CHECKERR (rsesci_SetScanRanges (io, 2));

            // Start
            CHECKERR (rsesci_SetScanStart (io, 1, 150.0e3));
            CHECKERR (rsesci_SetScanStart (io, 2, 30.0e6));
            // Stop
            CHECKERR (rsesci_SetScanStop (io, 1, 30.0e6));
            CHECKERR (rsesci_SetScanStop (io, 2, 1.0e9));
            // Step Size
            CHECKERR (rsesci_GetSweepSpacing (io, RSESCI_SCREEN_A, &stepMode));
    
            if (stepMode != RSESCI_AUTO)
                {
                CHECKERR (rsesci_SetScanStepSize (io, 1, 4.0e3));
                CHECKERR (rsesci_SetScanStepSize (io, 2, 40.0e3));
                }

            // Res BW
            CHECKERR (rsesci_SetScanBandwidth (io, 1, 9.0e3));
            CHECKERR (rsesci_SetScanBandwidth (io, 2, 120.0e3));
            // Meas Time
            CHECKERR (rsesci_SetScanMeasurementTime (io, 1, 0.001));
            CHECKERR (rsesci_SetScanMeasurementTime (io, 2, 0.0001));
            // Auto Ranging
            CHECKERR (rsesci_SetScanAutoRanging (io, 1, VI_FALSE));
            CHECKERR (rsesci_SetScanAutoRanging (io, 2, VI_FALSE));
            // RF Attn
            CHECKERR (rsesci_SetScanRFAttenuation (io, 1, 10.0));
            CHECKERR (rsesci_SetScanRFAttenuation (io, 2, 10.0));
            // Preamp
            CHECKERR (rsesci_SetScanPreamplifier (io, 1, VI_FALSE));
            CHECKERR (rsesci_SetScanPreamplifier (io, 2, VI_FALSE));
            // Auto Preamp
            CHECKERR (rsesci_GetScanAutoRanging (io, 1, &autoRanging));
            if (autoRanging == VI_TRUE)
                {
                CHECKERR (rsesci_SetScanAutoPreamplifier (io, 1, VI_FALSE));
                }
            //
            CHECKERR (rsesci_GetScanAutoRanging (io, 2, &autoRanging));
            if (autoRanging == VI_TRUE)
                {
                CHECKERR (rsesci_SetScanAutoPreamplifier (io, 2, VI_FALSE));
                }

            // --- Scan parameters
            
            if (domain == VI_TRUE)
                {
                CHECKERR (rsesci_GetStartFrequency (io, RSESCI_SCREEN_A, &xStart));
                CHECKERR (rsesci_GetStopFrequency (io, RSESCI_SCREEN_A, &xStop));
                }
            else
                {
                CHECKERR (rsesci_GetSweepTime (io, RSESCI_SCREEN_A, &xStart));
                CHECKERR (rsesci_GetScanTimeDomain (io, &xStop));
                }

            sprintf (buffer, "Start: %.1Le %s", xStart, (domain) ? "Hz" : "s");
        
            ReplaceAxisItem (panelHandle[MEAS], MEAS_GRAPH, VAL_TOP_XAXIS, 0, buffer, 0);

            // --- Single Scan
            CHECKERR (rsesci_SetScanMode (io, RSESCI_SCAN_SING));

            // ------ Prepare and Run Scan ------

            CHECKERR (rsesci_SetTraceFeedControl (io, 1, RSESCI_TRANS_ALWAYS));
            CHECKERR (rsesci_RunScan (io));

            traceStatus = 0;
            noofPoints = 0;
            points = 0;

            SetCtrlAttribute (panelHandle[MEAS], MEAS_TIMER, ATTR_ENABLED, 1);
            SetCtrlAttribute (panelHandle[MEAS], MEAS_ABORT_SCAN, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[MEAS], MEAS_GRAPH, ATTR_XLABEL_VISIBLE, 0);

            // Prepare xArray for pixel indexing (0,1,2,3...)
            for (idx = 0; idx < ARRAY_SIZE; idx++)
                xArray[idx] = idx;

            runState = 1;
            SetWaitCursor(0);
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","This button will configure the instrument to take a measurement.");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Upload Data                                                     */
/* Purpose:  This is a callback function driven by TIMER EVENTS.             */
/*===========================================================================*/
int  CVICALLBACK uploadData(int panel, int control, int event, void *callbackData,
                int eventData1, int eventData2)
{  
    switch (event) {
        case EVENT_TIMER_TICK:

            // ------ Data block transfer
            if (runState == 1)
                {
                ResetTimer (panelHandle[MEAS], MEAS_TIMER);

                // ------ All transferred blocks are collected and evaluated

                if (!(0x1000 & traceStatus)) // last of all blocks (for multiple scans after the last scan)
                    {
                    // Wait at least 250 milliseconds for block of data
                    // Delay (0.250);

                    if ((error = rsesci_ReadScanMeasurement (io, ARRAY_SIZE, &traceStatus, &noofPoints, tempStatusData, tempT1, tempT2, tempT3)) == VI_SUCCESS)
                        {
                        memcpy ((T1 + points), tempT1, noofPoints * sizeof (T1[0]));
                        memcpy ((T2 + points), tempT2, noofPoints * sizeof (T2[0]));
                        memcpy ((T3 + points), tempT3, noofPoints * sizeof (T3[0]));

                        points += noofPoints;

                        if (0x1 & traceStatus)
                            {
                            firstSubPoints = points;
                            }
                        else
                            {
                            secondSubPoints = points - firstSubPoints;
                            }

                        //printf ("- Taken Data:\nTrace Status 0x%x\nNumber of Points %ld\n", traceStatus, noofPoints);

                        DeleteGraphPlot (panelHandle[MEAS], MEAS_GRAPH, -1, VAL_DELAYED_DRAW);

                        PlotXY (panelHandle[MEAS], MEAS_GRAPH, xArray, T1, points, VAL_DOUBLE,
                                VAL_DOUBLE, VAL_THIN_LINE, VAL_SIMPLE_DOT, VAL_SOLID,
                                VAL_CONNECTED_POINTS, VAL_YELLOW);

                        PlotXY (panelHandle[MEAS], MEAS_GRAPH, xArray, T2, points, VAL_DOUBLE,
                                VAL_DOUBLE, VAL_THIN_LINE, VAL_SIMPLE_DOT, VAL_SOLID,
                                VAL_CONNECTED_POINTS, VAL_DK_GRAY);
                        
                        if (points > 1)
                            {
                            SetAxisScalingMode (panelHandle[MEAS], MEAS_GRAPH, VAL_BOTTOM_XAXIS,
                                                VAL_MANUAL, 0.0, points - 1);

                            SetAxisScalingMode (panelHandle[MEAS], MEAS_GRAPH, VAL_TOP_XAXIS,
                                                VAL_MANUAL, 0, points - 1);

                            sprintf (buffer, "Measured points: %ld", points);
                    
                            ReplaceAxisItem (panelHandle[MEAS], MEAS_GRAPH, VAL_TOP_XAXIS, 1,
                                             buffer, points - 1);
                            }
                        }
                    }
                else
                    {

                    // --- Final Data

                    sprintf (buffer, "Stop: %.1Le %s", xStop, (domain) ? "Hz" : "s");
                        
                    if (points > 1)
                        {
                        
                        if (domain) // Frequency Domain
                            {
                            CHECKERR (rsesci_Abort (io));
                            
                            CHECKERR (rsesci_GetScanStepSize (io, 1, &xIncrement));
                            CHECKERR (rsesci_GetScanStart (io, 1, &xSubStart));

                            for (idx = 0; idx < firstSubPoints; idx++)
                                xArray[idx] = idx * xIncrement + xSubStart;

                            CHECKERR (rsesci_GetScanStepSize (io, 2, &xIncrement));
                            CHECKERR (rsesci_GetScanStart (io, 2, &xSubStart));

                            for (idx = 0; idx < secondSubPoints; idx++)
                                xArray[idx + firstSubPoints] = idx * xIncrement + xSubStart;
                            }
                        else // Time Domain
                            {
                            xIncrement = (xStop - xStart) / points;

                            for (idx = 0; idx < points; idx++)
                                xArray[idx] = idx * xIncrement + xStart;
                            
                            }

                        DeleteGraphPlot (panelHandle[MEAS], MEAS_GRAPH, -1, VAL_DELAYED_DRAW);

                        SetAxisScalingMode (panelHandle[MEAS], MEAS_GRAPH, VAL_BOTTOM_XAXIS,
                                            VAL_MANUAL, xStart, xStop);

                        SetCtrlAttribute (panelHandle[MEAS], MEAS_GRAPH, ATTR_XLABEL_VISIBLE, 1);

                        PlotXY (panelHandle[MEAS], MEAS_GRAPH, xArray, T1, points, VAL_DOUBLE,
                                VAL_DOUBLE, VAL_THIN_LINE, VAL_SIMPLE_DOT, VAL_SOLID,
                                VAL_CONNECTED_POINTS, VAL_YELLOW);

                        PlotXY (panelHandle[MEAS], MEAS_GRAPH, xArray, T2, points, VAL_DOUBLE,
                                VAL_DOUBLE, VAL_THIN_LINE, VAL_SIMPLE_DOT, VAL_SOLID,
                                VAL_CONNECTED_POINTS, VAL_DK_GRAY);

                        ReplaceAxisItem (panelHandle[MEAS], MEAS_GRAPH, VAL_TOP_XAXIS, 1,
                                         buffer, points - 1);
                        }

                    SetCtrlAttribute (panelHandle[MEAS], MEAS_ABORT_SCAN, ATTR_DIMMED, 1);
                    SetCtrlAttribute (panelHandle[MEAS], MEAS_TIMER, ATTR_ENABLED, 0);
                    }

            ResetTimer (panelHandle[MEAS], MEAS_TIMER);
            }
        break;
    }
    
    return 0;
}

/*===========================================================================*/
/* Function: Aborts Scan                                                     */
/* Purpose:  Aborts currently runed scan.                                    */
/*===========================================================================*/
int  CVICALLBACK launchAbortScan(int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    
    SetWaitCursor(0);

    switch (event) {
        case EVENT_COMMIT:
            SetCtrlAttribute (panelHandle[MEAS], MEAS_TIMER, ATTR_ENABLED, 0);
            SetCtrlAttribute (panelHandle[MEAS], MEAS_ABORT_SCAN, ATTR_DIMMED, 1);

            traceStatus = 0;
            noofPoints = 0;
            points = 0;
            
            runState = 0;
            
            CHECKERR (rsesci_Abort (io));
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Control Help", "Aborts currently runed scan.");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Cancel                                                          */
/* Purpose:  Called by the init panel this function pops up a confirmation   */
/*           dialog box and then quits the user interface, if desired.       */
/*===========================================================================*/
int CVICALLBACK Cancel (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    SetWaitCursor(0);
    switch (event)
        {
        case EVENT_COMMIT:
            if ((ConfirmPopup ("Exit Application", "Are you sure you want"
                                                   " to quit this application?")) == 1)
                {
                memoryCleanup();
                QuitUserInterface (uirCode);
                }
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Control Help", "Closes the application.");
            break;
        }

    return 0;
}

/*===========================================================================*/
/* Function: Control Help                                                    */
/* Purpose:  This is a callback function of all controls that configure the  */
/*           instrument. On the right mouse-click on the control a help      */
/*           window describing its purpose is displayed.                     */
/*===========================================================================*/
int CVICALLBACK controlHelp (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{   
    SetWaitCursor(0);

    switch (event)
        {
        case EVENT_RIGHT_CLICK:
            
            if (panel == panelHandle[INIT])
                switch (control)
                    {
                    case INIT_ADDRESS:
                        MessagePopup ("Help", "This control specifies the interface and address of the device\n"
                                              "that is to be initialized (Instrument Descriptor). The exact\n"
                                              "grammar to be used in this control is shown in the note below.\n\n"
                                              "Default Value: GPIB::20\n\n"
                                              "Notes:\n\n"
                                              "(1) Based on the Instrument Descriptor, this operation\n"
                                              "establishes a communication session with a device.  The grammar\n"
                                              "for the Instrument Descriptor is shown below.  Optional\n"
                                              "parameters are shown in square brackets ([]).\n\n"
                                              "Interface   Grammar\n"
                                              "------------------------------------------------------\n"
                                              "Serial      ASRL<port>[::INSTR]\n"
                                              "GPIB        GPIB[board]::primary address[::secondary address]\n"
                                              "            [::INSTR]\n"
                                              "RSIB        RSIB::remote_host::INSTR\n\n"
                                              "The ASRL keyword is used for serial interface.\n"
                                              "The GPIB keyword is used for GPIB interface.\n"
                                              "The RSIB keyword is used for RSIB interface.\n\n"
                                              "Examples:\n"
                                              "(1) Serial - ASRL1\n"
                                              "(2) GPIB   - GPIB::20\n"
                                              "(3) RSIB   - RSIB::192.168.1.33::INSTR\n\n"
                                              "The default value for optional parameters are shown below.\n\n"
                                              "Optional Parameter          Default Value\n"
                                              "-----------------------------------------\n"
                                              "board                       0\n"
                                              "secondary address           none - 31\n\n"
                                              "Note:\n\n"
                                              "(1) Parameters of the serial interface, you must set via T&M\n"
                                              "Expolrer before you call initialize function. Default values are\n"
                                              "Baud Rate: 9600, Data Bits: 8, Parity: None, Stop Bits: 1 and\n"
                                              "Flow Control: None");
                        break;
                    case INIT_ID:
                        MessagePopup ("Help", "This control specifies if an ID Query is sent to the\n"
                                              "instrument during the initialization procedure.\n\n"
                                              "Valid Range:\n"
                                              "0 - Skip Query\n"
                                              "1 - Do Query (Default Value)");
                        break;
                    case INIT_RST:
                        MessagePopup ("Help", "This control specifies if the instrument is to be reset to its\n"
                                              "power-on settings during the initialization procedure.\n\n"
                                              "Valid Range:\n"
                                              "0 - Don't Reset\n"
                                              "1 - Reset Device (Default Value)");
                        break;
                    }

            if (panel == panelHandle[CONFIG])
                switch (control)
                    {
                    case CONFIG_DOMAIN:
                        MessagePopup ("Help", "Selects whether to run Time Domain Scan with Transfer of Block data (suitable for ESCI),\n"
                                              "or Frequency Domain Scan with Transfer of Block data (suitable for ESPI and ESCI).");
                        break;
                    case CONFIG_TRACE_DETECTOR_1:
                    case CONFIG_TRACE_DETECTOR_2:
                        MessagePopup ("Help", "This control switches on the detector for the data acquisition\n"
                                              "in the selected trace and the indicated measurement window.\n\n"
                                              "Valid Values:\n"
                                              "RSESCI_TRACEDET_NEG    (1) - Negative\n"
                                              "RSESCI_TRACEDET_POS    (2) - Positive\n"
                                              "RSESCI_TRACEDET_RMS    (4) - RMS\n"
                                              "RSESCI_TRACEDET_AVG    (5) - Average\n\n"
                                              "Note(s):\n\n"
                                              "(1) The 'Positive' or 'Negative' detector only displays the\n"
                                              "positive or the negative peak value.\n\n"
                                              "(2) The 'Average' detector displays the power average value at\n"
                                              "each test point.");
                        break;
                    case CONFIG_TRACE_MODE_1:
                    case CONFIG_TRACE_MODE_2:
                        MessagePopup ("Help", "This control defines the type of display and the evaluation of\n"
                                              "the traces in the selected measurement window.\n\n"
                                              "Valid Values:\n"
                                              "RSESCI_TRACE_WRITE (0) - Write\n"
                                              "RSESCI_TRACE_VIEW  (1) - View\n"
                                              "RSESCI_TRACE_MAXH  (3) - Max Hold\n"
                                              "RSESCI_TRACE_MINH  (4) - Min Hold\n\n"
                                              "Note(s):\n\n"
                                              "(1) 'Write' corresponds to the Clr/Write mode of manual operation.\n\n"
                                              "(2) The number of measurements for 'Average', 'Max Hold' and\n"
                                              "'Min Hold' is defined with the function rsesci_SetSweepCount().");
                        break;
                    }

            if (panel == panelHandle[MEAS]) 
                switch (control)
                    {
                    case MEAS_GRAPH:
                        MessagePopup ("Help", "Display of measurement results.");
                        break;
                    }

            break;
        }
    
    return 0;
}

/*===========================================================================*/
/* Function: Launch Configure Panel                                          */
/* Purpose:  This is a callback function of the button Configure on the      */
/*           panel Measure. It returns back to the Configuration panel to be */
/*           able to change configuration without having to re-run this      */
/*           program.                                                        */
/*===========================================================================*/
int CVICALLBACK launchConfig (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    SetWaitCursor(0);
    switch (event) {
        case EVENT_COMMIT:
            runState = 0;
            HidePanel (panelHandle[MEAS]);
            DisplayPanel (panelHandle[CONFIG]);
            ResetTimer (panelHandle[MEAS], MEAS_TIMER);
            SetCtrlAttribute (panelHandle[MEAS], MEAS_TIMER, ATTR_ENABLED, 0);
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","This button will recall the configuration panel.");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Panel Help                                                      */
/* Purpose:  This is a callback function of the menu bar. It displays a help */
/*           window describing panel being used.                             */
/*===========================================================================*/
void CVICALLBACK panelHelp (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    SetWaitCursor(0);

    if (panel == panelHandle[BCKGRND])
        MessagePopup ("Help", "This is a simple demo program using\n"
                              "the Rohde&Schwarz ESCI EMI Test Receiver Series\n"
                              "instrument driver provided in the package.\n");

    if (panel == panelHandle[INIT])
        MessagePopup ("Help", "This panel performs the following initialization actions:\n\n"
                              "- Opens a instrument session.\n\n"
                              "- Performs an identification query on the Instrument.\n\n"
                              "- Sends initialization commands to the instrument that set any\n"
                              "necessary programmatic variables to the state necessary\n"
                              "for the operation of the instrument driver.");
    
    if (panel == panelHandle[CONFIG])
        MessagePopup ("Help", "This panel configures the instrument.");
    
    if (panel == panelHandle[MEAS])
        MessagePopup ("Help", "This panel displays the measured values.\n"
                              "Configuration of the instrument is not\n"
                              "possible with this panel. Press the\n"
                              "\"Configure\" button to return to\n"
                              "the configuration panel.");

    return;
}

/*===========================================================================*/
/* Function: Launch Close Panel                                              */
/* Purpose:  This is a callback function of the button Close on the          */
/*           panel Measure. It pops-up a panel that will close the           */
/*           instrument.                                                     */
/*===========================================================================*/
int CVICALLBACK launchClose (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViChar errorMessage[256], errBuffer[256];

    SetWaitCursor(0);
    switch (event) {
        case EVENT_COMMIT:
            runState = 0;
            if ((ConfirmPopup ("Exit Application", "Are you sure you want to quit "
                    "this application?")) == 1) {
                if ((error = rsesci_close (io)) < 0) {
                    rsesci_error_message(VI_NULL, error, errorMessage);
                    sprintf (errBuffer, "Application failed to close the "
                        "instrument.\n\n%s\n\nExiting Application.", errorMessage);
                    MessagePopup("ERROR!", errBuffer);
                }

                memoryCleanup();
                QuitUserInterface (uirCode);
                exit (-1);
            }
            runState = 1;
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","This button will pop-up a panel to close the instrument.");
            break;
    }
    return 0;
}   

/*****************************************************************************/

long abortApplication (void)
{
    ViChar errorMessage[256], errBuffer[256];

    SetWaitCursor (0);
    rsesci_error_message(VI_NULL, error, errorMessage);
    sprintf (errBuffer,"The following error caused the application to fail:\n\n%s\n\n"
        "Attempting to Close Instrument Driver and Exit Application", 
        errorMessage);
    MessagePopup("ERROR!, Application Failed", errBuffer);
    
    if ((error = rsesci_close (io)) < 0) {
        rsesci_error_message(VI_NULL, error, errorMessage);
        sprintf (errBuffer, "Application failed to close the Instrument Driver.\n\n"
            "%s\n\nExiting Application.", errorMessage);
        MessagePopup("ERROR!",errBuffer);
    }
    else 
        MessagePopup("Application Aborted","Instrument Driver Closed, Exiting Application.");
        
    memoryCleanup();
    QuitUserInterface (uirCode);
    exit (-1);
    
    return 0;
}

/*****************************************************************************/

void memoryCleanup(void)
{
    free (tempStatusData);
    free (statusData);

    free (tempT1);
    free (T1);

    free (tempT2);
    free (T2);

    free (tempT3);
    free (T3);

    free (xArray);

    return;
}

/*= End =====================================================================*/
