/*

--- Scan with Transfer of Block data (suitable for ESPI and ESCI) ---

1. Programming of Scan parameters
      2 subscans
      use parameters of default setting, but program them to the unit
      scan start, scan stop, step mode
      subscan: start, stop, rbw, meas time, auto ranging, rf attn, (no preamp - is optional for ESPI)
      trace 1 clr write, detector peak; trace 2 clr write detector average single scan

2. Prepare scan
      enable srq with scan data available (STATus:OPERation) (workarounded)
      enable block transfer (TRACe:FEED:CONTrol ALWays)
      start scan

3. Data block transfer
      wait for SRQ (workarounded)
      read scan block
      evaluate level values from scan block
      display level values - needs to be discussed what is possible

--- Time Domain Scan with Transfer of Block data (suitable for ESCI) ---

1. Programming of Scan parameters
      select Time Domain
      measurement time 5 ms, total scan time 100 sec
      trace 1 clr write, detector peak; trace 2 clr write detector quasi peak
      single scan

2. Prepare scan
      enable srq with scan data available (STATus:OPERation) (workarounded)
      enable block transfer (TRACe:FEED:CONTrol ALWays)
      start scan

3. Data block transfer
      wait for SRQ (workarounded)
      read scan block
      evaluate level values from scan block
      display level values - needs to be discussed what is possible

*/

/****************************************************************************
 *                                                                          
 * Title:
 *
 * Purpose:
 *
 ****************************************************************************/

/* --- LabVINDOWS/CVI specific libraries --- */
#include <userint.h>
#include <utility.h>
/* --- Common C libraries, Instrument driver library, VISA library (included in instrument driver) --- */
#include <rsesci.h>
#include <ansi_c.h>

/* --- Macros --- */

#define CHECKERR( fCal ) \
    { \
        error = (fCal);\
        printf("Line %ld: %s\n", __LINE__, #fCal);\
        rsesci_error_message(io, error, error_message);\
        if (error < VI_SUCCESS) {\
            printf("\tPrimary Error: 0x%08X, ERROR: %s\n", error, error_message);\
            rsesci_error_query(io, &error, error_message);\
            printf("\tSecondary Error: %ld, %s\n", error, error_message);\
        } \
    }

/* --- Globals --- */

#define ARRAY_SIZE  100000 // Maximal number of measured frequencies is limited to 100000 (per detector)

/* --- Executable (Standalone executable) --- */
int main (int argc, char *argv[])
{
    ViStatus            error = VI_SUCCESS;
    ViChar              error_message[256];
    ViSession           io;
    
    /* Define remote connection parameters (as default values) */

    ViBoolean   IDQuery         = VI_TRUE,
                resetDevice     = VI_TRUE;
    ViRsrc      resourceName    = "RSIB::127.0.0.1"; /* "ASRL1", "GPIB::20", "RSIB::169.254.95.119" */

    /* Misc */

    ViInt32     stepMode        = 0;
    ViBoolean   autoRanging     = VI_FALSE;

    /* Specify either the CVI Standard Input/Output Window or a console window. */

    SetStdioPort (HOST_SYSTEM_STDIO);

    /* --- Init --- */

    CHECKERR (rsesci_init (resourceName, IDQuery, resetDevice, &io));
    CHECKERR (rsesci_SetDisplayUpdate (io, VI_TRUE));
    CHECKERR (rsesci_SetInstrumentMode (io, RSESCI_INSTRUMENT_MODE_RECEIVER));

    // ------ Programming of Scan parameters ------

    if (VI_TRUE)
        {
        // --- Scan in the Frequency Domain

        // Scan Start
        CHECKERR (rsesci_SetStartFrequency (io, RSESCI_SCREEN_A, 150.0e3));
        // Scan Stop
        CHECKERR (rsesci_SetStopFrequency (io, RSESCI_SCREEN_A, 1.0e9));
        // Step Mode
        CHECKERR (rsesci_SetSweepSpacing (io, RSESCI_SCREEN_A, RSESCI_AUTO));
        }
    else
        {
        // --- Scan in the Time Domain

        // Set fixed frequency mode
        CHECKERR (rsesci_SetFrequencyMode (io, RSESCI_SCREEN_A, RSESCI_FREQ_FIXED));
        // Set Total Scan Time
        CHECKERR (rsesci_SetScanTimeDomain (io, 100.0));
        // Set Receiver Frequency
        CHECKERR (rsesci_SetCenterFrequency (io, RSESCI_SCREEN_A, 1.0e9));
        // Set Measurement Time
        CHECKERR (rsesci_SetSweepTime (io, RSESCI_SCREEN_A, 5.0e-3));
        }

    // Trace Parameters
    CHECKERR (rsesci_SetTraceMode (io, RSESCI_SCREEN_A, 1, RSESCI_TRACE_WRITE));
    CHECKERR (rsesci_SetTraceDetector (io, RSESCI_SCREEN_A, 1, RSESCI_TRACEDET_POS));
    //
    CHECKERR (rsesci_SetTraceMode (io, RSESCI_SCREEN_A, 2, RSESCI_TRACE_WRITE));
    CHECKERR (rsesci_SetTraceDetector (io, RSESCI_SCREEN_A, 2, RSESCI_TRACEDET_AVG));

    // --- Scan Table ---

    CHECKERR (rsesci_SetScanRanges (io, 2));

    // Start
    CHECKERR (rsesci_SetScanStart (io, 1, 150.0e3));
    CHECKERR (rsesci_SetScanStart (io, 2, 30.0e6));
    // Stop
    CHECKERR (rsesci_SetScanStop (io, 1, 30.0e6));
    CHECKERR (rsesci_SetScanStop (io, 2, 1.0e9));
    // Step Size
    CHECKERR (rsesci_GetSweepSpacing (io, RSESCI_SCREEN_A, &stepMode));
    
    if (stepMode != RSESCI_AUTO)
        {
        CHECKERR (rsesci_SetScanStepSize (io, 1, 4.0e3));
        CHECKERR (rsesci_SetScanStepSize (io, 2, 40.0e3));
        }

    // Res BW
    CHECKERR (rsesci_SetScanBandwidth (io, 1, 9.0e3));
    CHECKERR (rsesci_SetScanBandwidth (io, 2, 120.0e3));
    // Meas Time
    CHECKERR (rsesci_SetScanMeasurementTime (io, 1, 0.001));
    CHECKERR (rsesci_SetScanMeasurementTime (io, 2, 0.0001));
    // Auto Ranging
    CHECKERR (rsesci_SetScanAutoRanging (io, 1, VI_FALSE));
    CHECKERR (rsesci_SetScanAutoRanging (io, 2, VI_FALSE));
    // RF Attn
    CHECKERR (rsesci_SetScanRFAttenuation (io, 1, 10.0));
    CHECKERR (rsesci_SetScanRFAttenuation (io, 2, 10.0));
    // Preamp
    CHECKERR (rsesci_SetScanPreamplifier (io, 1, VI_FALSE));
    CHECKERR (rsesci_SetScanPreamplifier (io, 2, VI_FALSE));
    // Auto Preamp
    CHECKERR (rsesci_GetScanAutoRanging (io, 1, &autoRanging));
    if (autoRanging == VI_TRUE)
        {
        CHECKERR (rsesci_SetScanAutoPreamplifier (io, 1, VI_FALSE));
        }
    //
    CHECKERR (rsesci_GetScanAutoRanging (io, 2, &autoRanging));
    if (autoRanging == VI_TRUE)
        {
        CHECKERR (rsesci_SetScanAutoPreamplifier (io, 2, VI_FALSE));
        }
    // Single Scan
    CHECKERR (rsesci_SetScanMode (io, RSESCI_SCAN_SING));
    
    // ------ Prepare Scan ------

    CHECKERR (rsesci_SetTraceFeedControl (io, 1, RSESCI_TRANS_ALWAYS));
    CHECKERR (rsesci_RunScan (io));

    // ------ Data block transfer ------
    
    {   
    ViInt32     traceStatus     = 0,
                noofPoints      = 0;

    ViChar      *statusData, *tempStatusData;
    ViReal64    *T1, *tempT1;
    ViReal64    *T2, *tempT2;
    ViReal64    *T3, *tempT3;

    // --- Memory alloc

    tempStatusData = malloc (ARRAY_SIZE * sizeof (ViChar));
    statusData = malloc (ARRAY_SIZE * sizeof (ViChar));

    T1 = malloc (ARRAY_SIZE * sizeof (ViReal64));
    tempT1 = malloc (ARRAY_SIZE * sizeof (ViReal64));

    T2 = malloc (ARRAY_SIZE * sizeof (ViReal64));
    tempT2 = malloc (ARRAY_SIZE * sizeof (ViReal64));

    T3 = malloc (ARRAY_SIZE * sizeof (ViReal64));
    tempT3 = malloc (ARRAY_SIZE * sizeof (ViReal64));

    // --- All transferred blocks are collected and evaluated

    traceStatus = 0x0;
    noofPoints = 0;
    
    while (!(0x1000 & traceStatus)) // last of all blocks (for multiple scans after the last scan)

        {
        // Wait at least 250 milliseconds for block of data
        Delay (0.250);

        CHECKERR (rsesci_ReadScanMeasurement (io, ARRAY_SIZE, &traceStatus, &noofPoints, tempStatusData, tempT1, tempT2, tempT3));
        
        printf ("- Taken Data:\nTrace Status 0x%x\nNumber of Points %ld\n", traceStatus, noofPoints);

        }

    // --- Memory Cleanup

    free (tempStatusData);
    free (statusData);

    free (tempT1);
    free (T1);

    free (tempT2);
    free (T2);

    free (tempT3);
    free (T3);

    }

    /* --- Close --- */

    CHECKERR (rsesci_close (io));

    return 0;
}
